<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Laskuhari Gateway.
 *
 * Provides a Payment Gateway for Laskuhari invoicing.
 * Edited from WC_Gateway_COD by Datahari Solutions
 *
 * @class 		WC_Gateway_Laskuhari
 * @extends		WC_Payment_Gateway
 * @version		0.9.11
 */
class WC_Gateway_Laskuhari extends WC_Payment_Gateway {

    /**
     * Constructor for the gateway.
     */
	public function __construct($notexts = false) {
		$this->id                 		= 'laskuhari';
		$this->icon               		= apply_filters( 'woocommerce_laskuhari_icon', '' );
		$this->method_title       		= __( 'Laskuhari', 'woocommerce' );
		$this->method_description 		= __( 'Käytä Laskuhari-palvelua tilausten automaattiseen laskuttamiseen', 'woocommerce' );
		$this->has_fields         		= false;

		// Load the settings
		$this->init_form_fields();
		$this->init_settings();

		// Get settings
		$this->title              		= $this->get_option( 'title' );
		$this->uid                		= $this->get_option( 'uid' );
		$this->apikey             		= $this->get_option( 'apikey' );
		$this->lahetystapa_manuaalinen  = $this->get_option( 'lahetystapa_manuaalinen' );
		$this->lahetystapa        		= $this->get_option( 'lahetystapa' );
		$this->laskuviesti        		= $this->get_option( 'laskuviesti' );
		$this->laskuttaja         		= $this->get_option( 'laskuttaja' );
		$this->description        		= $this->get_option( 'description' );
		$this->instructions       		= $this->get_option( 'instructions', $this->description );
		$this->enable_for_methods 		= $this->get_option( 'enable_for_methods', array() );
		$this->enable_for_virtual 		= $this->get_option( 'enable_for_virtual', 'yes' ) === 'yes' ? true : false;

		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		if($notexts == false) {
			add_action( 'woocommerce_thankyou_laskuhari', array( $this, 'thankyou_page' ) );
	    	// Customer Emails
	    	add_action( 'woocommerce_email_before_order_table', array( $this, 'email_instructions' ), 10, 3 );
		}
	}

    /**
     * Initialise Gateway Settings Form Fields.
     */
    public function init_form_fields() {
    	$shipping_methods = array();

    	if ( is_admin() )
	    	foreach ( WC()->shipping()->load_shipping_methods() as $method ) {
		    	$shipping_methods[ $method->id ] = $method->get_title();
	    	}

    	$this->form_fields = array(
			'enabled' => array(
				'title'       => __( 'Käytössä', 'woocommerce' ),
				'label'       => __( 'Ota käyttöön Laskuhari-lisäosa', 'woocommerce' ),
				'type'        => 'checkbox',
				'description' => '',
				'default'     => 'no'
			),
			'gateway_enabled' => array(
				'title'       => __( 'Maksutapa', 'woocommerce' ),
				'label'       => __( 'Ota käyttöön Laskutus-maksutapa', 'woocommerce' ),
				'type'        => 'checkbox',
				'description' => 'Lisää verkkokauppaan Laskutus-maksutavan, joka lähettää asiakkaalle tilauksesta laskun automaattisesti.',
				'default'     => 'yes'
			),
			'uid' => array(
				'title'       => __( 'UID', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Laskuhari-tunnuksesi UID (kysy asiakaspalvelusta)', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'apikey' => array(
				'title'       => __( 'API-koodi', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Laskuhari-tunnuksesi API-koodi (kysy asiakaspalvelusta)', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'lahetystapa' => array(
				'title'       => __( 'Lähetystapa (automaattinen)', 'woocommerce' ),
				'label'       => __( 'Valitse laskujen lähetystapa', 'woocommerce' ),
				'type'        => 'select',
				'description' => __( 'Valitse tapa, jolla haluat lähettää automaattisesti lähettävät laskut', 'woocommerce' ),
				'default'     => 'email',
				'options'     => array(
					'email' => __( 'Sähköpostilasku', 'woocommerce' ),
					'kirje'   => __( 'Kirjelasku', 'woocommerce' ),
					'auto'   => __( 'Kirje, mikäli sähköpostiosoitetta ei annettu', 'woocommerce' ),
					'ei'   => __( 'Tallenna Laskuhariin, älä lähetä', 'woocommerce' )
				)
			),
			'lahetystapa_manuaalinen' => array(
				'title'       => __( 'Lähetystapa (manuaalinen)', 'woocommerce' ),
				'label'       => __( 'Valitse laskujen lähetystapa', 'woocommerce' ),
				'type'        => 'select',
				'description' => __( 'Valitse tapa, jolla haluat lähettää manuaalisesti lähettävät laskut', 'woocommerce' ),
				'default'     => 'email',
				'options'     => array(
					'email' => __( 'Sähköpostilasku', 'woocommerce' ),
					'kirje'   => __( 'Kirjelasku', 'woocommerce' ),
					'auto'   => __( 'Kirje, mikäli sähköpostiosoitetta ei annettu', 'woocommerce' ),
					'ei'   => __( 'Tallenna Laskuhariin, älä lähetä', 'woocommerce' )
				)
			),
			'laskuviesti' => array(
				'title'       => __( 'Laskuviesti', 'woocommerce' ),
				'type'        => 'textarea',
				'description' => __( 'Viesti, joka lähetetään saatetekstinä sähköpostilaskun ohessa', 'woocommerce' ),
				'default'     => __( 'Hei,'."\n\n".'Kiitos tilauksestasi. Liitteenä lasku tilaamistasi tuotteista.', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'laskuttaja' => array(
				'title'       => __( 'Laskuttaja', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Laskuttajan nimi, joka näkyy sähköpostilaskun lähettäjänä', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'title' => array(
				'title'       => __( 'Otsikko', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Tämä näkyy maksutavan nimenä asiakkaalle', 'woocommerce' ),
				'default'     => __( 'Laskutus', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'description' => array(
				'title'       => __( 'Kuvaus', 'woocommerce' ),
				'type'        => 'textarea',
				'description' => __( 'Kuvaus, joka näytetään maksutavan yhteydessä', 'woocommerce' ),
				'default'     => __( 'Maksa tilauksesi kätevästi laskulla', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'instructions' => array(
				'title'       => __( 'Ohjeet', 'woocommerce' ),
				'type'        => 'textarea',
				'description' => __( 'Ohjeet, jotka näkyvät tilausvahvistussivulla', 'woocommerce' ),
				'default'     => __( 'Lähetämme sinulle laskun tilauksestasi.', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'enable_for_methods' => array(
				'title'             => __( 'Käytössä näille toimitustavoille', 'woocommerce' ),
				'type'              => 'multiselect',
				'class'             => 'wc-enhanced-select',
				'css'               => 'width: 450px;',
				'default'           => '',
				'description'       => __( 'Jätä tyhjäksi, jos haluat laskutuksen käyttöön kaikille toimitustavoille', 'woocommerce' ),
				'options'           => $shipping_methods,
				'desc_tip'          => true,
				'custom_attributes' => array(
					'data-placeholder' => __( 'Valitse toimitustavat', 'woocommerce' )
				)
			),
			'enable_for_virtual' => array(
				'title'             => __( 'Virtuaalituotteet', 'woocommerce' ),
				'label'             => __( 'Hyväksy laskutus-maksutapa, jos tuote on virtuaalinen', 'woocommerce' ),
				'type'              => 'checkbox',
				'default'           => 'yes'
			)
 	   );
    }

	/**
	 * Check If The Gateway Is Available For Use.
	 *
	 * @return bool
	 */
	public function is_available() {
		if($this->get_option('gateway_enabled') == 'no') {
			return false;
		}
		$order          = null;
		$needs_shipping = false;

		// Test if shipping is needed first
		if ( WC()->cart && WC()->cart->needs_shipping() ) {
			$needs_shipping = true;
		} elseif ( is_page( wc_get_page_id( 'checkout' ) ) && 0 < get_query_var( 'order-pay' ) ) {
			$order_id = absint( get_query_var( 'order-pay' ) );
			$order    = wc_get_order( $order_id );

			// Test if order needs shipping.
			if ( 0 < sizeof( $order->get_items() ) ) {
				foreach ( $order->get_items() as $item ) {
					$_product = $order->get_product_from_item( $item );
					if ( $_product && $_product->needs_shipping() ) {
						$needs_shipping = true;
						break;
					}
				}
			}
		}

		$needs_shipping = apply_filters( 'woocommerce_cart_needs_shipping', $needs_shipping );

		// Virtual order, with virtual disabled
		if ( ! $this->enable_for_virtual && ! $needs_shipping ) {
			return false;
		}

		// Check methods
		if ( ! empty( $this->enable_for_methods ) && $needs_shipping ) {

			// Only apply if all packages are being shipped via chosen methods, or order is virtual
			$chosen_shipping_methods_session = WC()->session->get( 'chosen_shipping_methods' );

			if ( isset( $chosen_shipping_methods_session ) ) {
				$chosen_shipping_methods = array_unique( $chosen_shipping_methods_session );
			} else {
				$chosen_shipping_methods = array();
			}

			$check_method = false;

			if ( is_object( $order ) ) {
				if ( $order->shipping_method ) {
					$check_method = $order->shipping_method;
				}

			} elseif ( empty( $chosen_shipping_methods ) || sizeof( $chosen_shipping_methods ) > 1 ) {
				$check_method = false;
			} elseif ( sizeof( $chosen_shipping_methods ) == 1 ) {
				$check_method = $chosen_shipping_methods[0];
			}

			if ( ! $check_method ) {
				return false;
			}

			$found = false;

			foreach ( $this->enable_for_methods as $method_id ) {
				if ( strpos( $check_method, $method_id ) === 0 ) {
					$found = true;
					break;
				}
			}

			if ( ! $found ) {
				return false;
			}
		}

		return parent::is_available();
	}


    /**
     * Process the payment and return the result.
     *
     * @param int $order_id
     * @return array
     */
	public function process_payment( $order_id ) {

		$lh = laskuhari_process_action($order_id, false, wc_get_order($order_id));
        $order      = $lh['order'];
        $notice     = $lh['notice'];

		// Reduce stock levels
		$order->reduce_order_stock();

		// Remove cart
		WC()->cart->empty_cart();

		// Return thankyou redirect
		return array(
			'result' 	=> 'success',
			'redirect'	=> $this->get_return_url( $order )
		);
	}

    /**
     * Output for the order received page.
     */
	public function thankyou_page() {
		if ( $this->instructions ) {
        	echo wpautop( wptexturize( $this->instructions ) );
		}
	}

    /**
     * Add content to the WC emails.
     *
     * @access public
     * @param WC_Order $order
     * @param bool $sent_to_admin
     * @param bool $plain_text
     */
	public function email_instructions( $order, $sent_to_admin, $plain_text = false ) {
		if ( $this->instructions && ! $sent_to_admin && 'laskuhari' === $order->payment_method ) {
			echo wpautop( wptexturize( $this->instructions ) ) . PHP_EOL;
		}
	}
}
